package trz.gaia.rna;

import trz.gaia.*;
import java.util.*;

/**
Una classe che gestisce l' effettiva spedizione dei pacchetti da una cellula
all' altra.<br>
L' eventuale simulazione della perdita di pacchetti viene gestita a livello di
connessione.<br>
*/
//Impostato come un singleton.
//TEMP: public solo per i javadoc
public class LinkLayer {

	private static LinkLayer istanza;

	private LinkLayer(){}
	
	public static LinkLayer getInstance(){
		if (istanza == null)
			istanza = new LinkLayer();
		return istanza;
	}
	
	//ogni con all' interno e' individuata dalla concatenazione di nomeFrom - nomeTo
	HashMap connessioni = new HashMap();
	/**
	Gestisce l' handshake sul link layer e restituisce una connessione se va tutto bene,
	null altrimenti
	*/
	//### trovare un modo migliore: e' facile sbagliare o barare
	public Connessione getConnessione(Cellula cellulaFrom, String nomeTo) {
		
		//creo una connessione locale
		/*
		mando la richiesta all' altra cellula (handshake in due tempi; SYN e ACK e basta)
		###Qui e' un po' brutto che usi i nomi delle cellule anche sul link layer, ma pace :(
		*/
		LinkPacket lpckt = new LinkPacket();
		lpckt.setTipo(LinkPacket.SYN);
		put(lpckt, nomeTo);
		//attendo la risposta
		while(true) {	//###mettere un timeout
			
			LinkPacket resLpckt = cellulaFrom.fetchPacket();
			if (resLpckt != null) {
				if (resLpckt.getTipo() == LinkPacket.ACK) {
					//tutto ok
					Connessione con = new Connessione(cellulaFrom, nomeTo);
					//mi salvo la con
					connessioni.put(cellulaFrom.getNome() + "-" + nomeTo, con);
					return con;
				}
				else {
					System.out.println("Con rifiutata: " + resLpckt.getTipo());
					return null;
				}
			}//fine lpckt!=null
		}//fine while
	}

	/**
	Aggiunge un pckt a livello di link layer
	*/
	public void put(LinkPacket lpckt, String nomeTo) {
		
		Cellula cellulaTo = listaCellule.get(nomeTo);
		cellulaTo.putPacket(lpckt);
	}

	/**
	Scrive un pckt a livello di network layer
	*/
	public void write(Packet pckt) {
		
		Connessione con = (Connessione)connessioni.get(pckt.getNomeTo()+"-"+pckt.getNomeFrom());
		if (con == null)
			//###generare un exception
			return;
		con.putPacket(pckt);
	}

	/*
	Non c'e' un metodo read perche' quando spedisco un pacchetto questo viene caricato
	direttamente nel buffer del destinatario.
	L' alternativa era che ogni cellula chiedesse al link layer i suoi pacchetti, ma questo 
	avrebbe voluto dire che, piu' o meno volontariamente, una cellula avrebbe potuto "sniffare"
	pacchetti altrui, e la cosa nin mi piaceva molto.
	*/

}
