package trz.server;

import java.io.*;
import java.net.*;

public class Server {
   
	protected int porta = 11277;

	protected int maxThreads = 3;
	protected	ThreadGroup connessioni = new ThreadGroup("Connessioni");
 
	public void setPorta(int porta) {
		this.porta = porta;
	}

	public int getPorta() {
		return porta;		
	}

	public void setMaxCon(int maxCon) {
		maxThreads = maxCon;
	}

	public int getMaxCon() {
		return maxThreads;
	}

	public static void main(String[] args) {
		
		Server srv = new Server();
		srv.avvia();
	}
	
	public void avvia() {

    System.out.println("Porta: " + porta);
    System.out.println("MaxThread: " + maxThreads);
    
	Socket s = null;
    ServerSocket ss = null;
	
    try {
      ss = new ServerSocket(porta);
    }catch (IOException e){
      System.out.println("Errore: impossibile avviare il server - " + e);
		System.exit(1);
    }

    while (true) {
  		try {
		//qui dovrei limitare il numero di con. Il problema e' che una volta che uno ha creato un serverSocket
		//tutti sono convinti di riuscire a connettersi su quella porta, anche se uno non e' un accept :(	
        System.out.println("In attesa (" + connessioni.activeCount() +"/"+ maxThreads + ")...");
        s = ss.accept();
				synchronized (connessioni) {
					connessioni.list();
					if (connessioni.activeCount() < maxThreads)
						accettaConnessione(s, connessioni);
					else {
						rifiutaConnessione(s);
					}
				}
  		}catch (IOException e){
    		System.out.println("Errore: problema durante la connessione - " + e);
	  	}
		}	//fine while
  }

	protected void accettaConnessione(Socket s, ThreadGroup connessioni) {

		System.out.println(connessioni.activeCount() + ": connesso con " + s.getLocalAddress());
		//chiudiConnessione(s);
		try {
			s.getOutputStream().write(0);
		}catch(IOException e){
			System.out.println("Errore durante la segnalazione di ok " + e);
		}
		Ascoltatore a = new Ascoltatore(connessioni);
		a.setSocket(s);
		a.start();		
	}

	protected void rifiutaConnessione(Socket s) {

		System.out.println("Connessione con " + s.getLocalAddress() + " rifiutata: troppi accessi. Max = " + maxThreads);
		try {
			s.getOutputStream().write(1);
		}catch(IOException e){
			System.out.println("Errore durante la segnalazione di rifiuto " + e);
		}
		chiudiConnessione(s);
	}

	protected void chiudiConnessione(Socket s) {
		
		if (s != null) {
			String indirizzo = s.getLocalAddress().toString();
			try {
				s.close();
				System.out.println("Chiusa connessione con " + indirizzo);
			}catch (IOException e){
	  		System.out.println("Errore: impossibile chiudere il socket con " + indirizzo + " - " + e);
			}
		}
	}

};
